//postavljamo datoteke koje integriraju trazene komponente
#include <ESP32Servo.h>
#include <LoRa.h>

//postavljamo vrijednosti pinova LoRa modula
#define ss 5
#define rst 13
#define dio0 4

//postavljamo globalne varijable
const uint8_t motorEscPin [] = {27 , 26};               //pinovi esc kontrolera
short motorSpeedSet[] = {0, 0}, motorSpeed[] = {0, 0};  //motorSpeedSet-postavljanje brzina motora kako bi interpolirali premo to vrijednosti te ju postavljali na-motorSpeed- nizu
long lastSent = 0, speedSetTimePass = 0;                //lastSent-proteklo vrijeme od posljednje poruke, speedSetTimePass-brojaci pracenje vremene koje je proslo

//deklariranje esc kao servo motor
Servo esc[2];

//integriranje pocetne methode
void setup() {
  //Serial.begin(115200);

  //alokacija svih PWM timera
  ESP32PWM::allocateTimer(0);
  ESP32PWM::allocateTimer(1);
  ESP32PWM::allocateTimer(2);
  ESP32PWM::allocateTimer(3);

  //integriranje svih esc kontrolera
  for (int i = 0; i < 2; i++) {
    esc[i].setPeriodHertz(50);
    esc[i].attach(motorEscPin[i], 1100, 1900);
    esc[i].writeMicroseconds(1500);
  }

  //integriranje LoRa modula
  LoRa.setPins(ss, rst, dio0);

  //LoRa boot
  while (!LoRa.begin(866E6)) {
    delay(500);
  }

  //LoRa moduli prefix poruke, povezevanje dva modula
  LoRa.setSyncWord(0x28);
}

//integriranje methode koja se konstantno ponavlja poslje setup methode
void loop() {
  //restiranje svih vrijednosti motora na nulu ukoliko je od zadnje poruke proslo vise od 10 sekundi
  if (millis() > lastSent + 10000) {
    motorSpeedSet[0] = 0;
    motorSpeedSet[1] = 0;
  }

  //uvjet interpoliranje brzine motora
  if (millis() > speedSetTimePass + 5) {
    speedSetTimePass = millis();

    //interpoliranje pojedinih motora
    for (int i = 0; i < 2; i++) {
      if (motorSpeed[i] > motorSpeedSet[i])
        motorSpeed[i]--;
      else if (motorSpeed[i] < motorSpeedSet[i])
        motorSpeed[i]++;

      int val = 1500;
      if (motorSpeed[i] < 0)
        val = map(motorSpeed[i], -255, 0, 1100, 1400);
      else if (motorSpeed[i] > 0)
        val = map(motorSpeed[i], 0, 255, 1600, 1900);
      esc[i].writeMicroseconds(val);
    }
  }

  //Serial.print("L:"); Serial.print(motorSpeed[0]); Serial.print(" R:"); Serial.println(motorSpeed[1]);

  //primanje poruke
  int packetSize = LoRa.parsePacket();
  if (packetSize) {
    lastSent = millis();

    //ocitavanje vrijednosti poruke
    while (LoRa.available()) {
      //Serial.print(LoRa.packetRssi());
      String loraPacket = LoRa.readString();
      char type[15] = {0}, val[31] = {0};     //type-ime vrijednosti, val-vrijednost
      long cnt = 0;                           //brojac
      bool gotType = false;                   //vrijednost pronadjena da/ne

      //ocitivanje pojedinih vrijednosti poruke -> vrijednost L2 i R2 
      for (int i = 0; i < loraPacket.length(); i++) {
        if (loraPacket[i] == 0) break;
        if (loraPacket[i] != ':' && !gotType)
          type[cnt] = loraPacket[i];
        else {
          if (!gotType) {
            gotType = true;
            cnt = 0;
            continue;
          }
          if (loraPacket[i] != ';')
            val[cnt] = loraPacket[i];
          else {

            //postavljanje brzine koje su ocitane iz poruke
            if (type[0] == 'L') {
              motorSpeedSet[0] = atoi(val);
              if (motorSpeedSet[0] > 255)
                motorSpeedSet[0] = 255;
              else if (motorSpeedSet[0] < -255)
                motorSpeedSet[0] = -255;
            } else if (type[0] == 'R') {
              motorSpeedSet[1] = atoi(val);
              if (motorSpeedSet[1] > 255)
                motorSpeedSet[1] = 255;
              else if (motorSpeedSet[1] < -255)
                motorSpeedSet[1] = -255;
            }

            //resetiranje svih lokalnih varijabli
            if (i != sizeof(loraPacket) - 1) {
              cnt = 0;
              gotType = false;
              for (int n = 0; n < sizeof(type); n++) {
                type[n] = 0;
              }
              for (int n = 0; n < sizeof(val); n++) {
                val[n] = 0;
              }
            }
            continue;
          }
        }
        ++cnt;
      }
    }
  }

}
