//postavljamo datoteke koje integriraju trazene komponente
#include <PS4Controller.h>
#include "esp_bt_remove_devices.h"
#include <LoRa.h>

//postavljamo vrijednosti pinova LoRa modula
#define ss 5
#define rst 13
#define dio0 4

//postavljamo globalne varijable
long millisPass[3] = {0};                                       //brojaci pracenje vremene koje je proslo
bool reverseMode = false, sendNull[] = {false, false, false};   //reverseMode-brod se krece u nazad da/ne, sendNull-je li brzina motora postavljena na nulu
short motorDir[] = {1, 1};                                      //smjer rotacije pojedinog motora

//postavljanje poruku za lora modul lijevog motora
void loraPrintL(int val) {
  LoRa.print("L:");
  LoRa.print(val);
  LoRa.print(';');
}

//postavljanje poruku za lora modul desnog motora
void loraPrintR(int val) {
  LoRa.print("R:");
  LoRa.print(val);
  LoRa.print(';');
}

//integriranje pocetne methode
void setup() {
  //Serial.begin(115200);

  //resetrianje bluetooth buffera esp32
  bt_remove_paired_devices();
  //integriranje ps4 joysticka na MAC dedicated mac adresi
  PS4.begin("e0:cc:f8:e3:2c:c9");

  //integriranje LoRa modula
  LoRa.setPins(ss, rst, dio0);

  //LoRa boot
  while (!LoRa.begin(866E6)) {
    //Serial.println(".");
    delay(500);
  }

  //LoRa moduli prefix poruke, povezevanje dva modula
  LoRa.setSyncWord(0x28);
}

//integriranje methode koja se konstantno ponavlja poslje setup methode
void loop() {
  //uvjet aktivan ukoliko je ps4 kontroler spojen
  if (PS4.isConnected()) {
    //ako je omoguceno kretanje u suprotnom smjeru i strelica prema dolje pritisnuta, motor mjenja smjer kretanje
    if (PS4.Down() && reverseMode)
      motorDir[0] = -1;
    else
      motorDir[0] = 1;

    //ako je omoguceno kretanje u suprotnom smjeru i kriz pritisnuta, motor mjenja smjer kretanje
    if (PS4.Cross() && reverseMode)
      motorDir[1] = -1;
    else
      motorDir[1] = 1;

    //aktiviranje ili deaktiviranje kretanja broda u nazad
    if (PS4.Share() && PS4.Options()) {
      if (millis() > millisPass[0] + 2000) {
        if (reverseMode) {
          if (PS4.Battery() > 2) PS4.setLed(255, 255, 255);
          for (int n = 0; n < 2; n++) {
            motorDir[n] = 1;
          }
          reverseMode = false;
        } else {
          PS4.setLed(255, 0, 0);
          reverseMode = true;
        }
        millisPass[0] = millis();
        PS4.sendToController();
      }
    } else
      millisPass[0] = millis();

    //ukoliko je baterija prazna aktivira se statusno svjetlo na ps4 kontroleru
    if (PS4.Battery() < 3) {
      PS4.setFlashRate(1000, 1000);
      PS4.setLed(255, 0, 0);
      if (millisPass[1] >= millis() + 10000) {
        PS4.sendToController();
        millisPass[1] = millis();
      }
    } else
      PS4.setFlashRate(0, 0);

    //uvjet ukoliko je R2 ili L2 pritisnut
    if (!PS4.R2() && !PS4.L2()) {
      millisPass[2] = millis();
      if (!sendNull[0]) {
        sendNull[0] = true;
        LoRa.beginPacket();
        loraPrintL(0);
        loraPrintR(0);
        LoRa.endPacket();
      }
    }
    else {
      sendNull[0] = false;
      if (millis() > millisPass[2] + 10) {
        millisPass[2] = millis();
        LoRa.beginPacket();

        //ukoliko je L2 pritisnut postavljamo poruku za LoRa modul
        if (PS4.L2()) {
          sendNull[1] = false;
          loraPrintL(PS4.L2Value()*motorDir[0]);
        } else if (!sendNull[1]) {
          sendNull[1] = true;
          loraPrintL(0);
        }

        //ukoliko je R2 pritisnut postavljamo poruku za LoRa modul
        if (PS4.R2()) {
          sendNull[2] = false;
          loraPrintR(PS4.R2Value()*motorDir[1]);
        } else if (!sendNull[2]) {
          sendNull[2] = true;
          loraPrintR(0);
        }
        LoRa.endPacket();
      }
    }
  }
}
